@@ -35,6 +35,7 @@ const Application = imports.application;
 const MapView = imports.mapView;
 const SearchPopup = imports.searchPopup;
 const ContextMenu = imports.contextMenu;
+const PlaceStore = imports.placeStore;
 const Utils = imports.utils;
 const Config = imports.config;
 
@@ -49,7 +50,7 @@ const _PLACE_ICON_SIZE = 20;
 const SearchResults = {
     COL_ICON:         0,
     COL_DESCRIPTION:  1,
-    COL_LOCATION:     2
+    COL_PLACE:        2
 };
 
 const MainWindow = new Lang.Class({
@@ -73,6 +74,7 @@ const MainWindow = new Lang.Class({
 
         this._contextMenu = new ContextMenu.ContextMenu(this.mapView);
 
+        this._initPlaces();
         this._initSearchWidgets();
         this._initActions();
         this._initSignals();
@@ -85,6 +87,16 @@ const MainWindow = new Lang.Class({
         grid.show_all();
     },
 
+    _initPlaces: function() {
+        this._placeStore = new PlaceStore.PlaceStore();
+        try {
+            this._placeStore.load();
+        } catch (e) {
+            log('Failed to parse Maps places file, ' +
+                'subsequent writes will overwrite the file!');
+        }
+    },
+
     _initSearchWidgets: function() {
         this._searchPopup = new SearchPopup.SearchPopup(10);
 
@@ -99,6 +111,28 @@ const MainWindow = new Lang.Class({
                                   this._searchPopup.hide.bind(this._searchPopup));
         this._searchEntry.connect('changed',
                                   this._searchPopup.hide.bind(this._searchPopup));
+
+        // FIXME: do as much of the following from UI file as possible
+        let completion = new Gtk.EntryCompletion({ model: this._placeStore,
+                                                   popup_completion: true,
+                                                   inline_completion: false,
+                                                   minimum_key_length: 2 });
+
+        let renderer = new Gtk.CellRendererPixbuf({ xpad: 2 });
+        completion.pack_start(renderer, false);
+        completion.add_attribute(renderer, "pixbuf", PlaceStore.Columns.PLACE_ICON);
+        completion.reorder(renderer, 0);
+        completion.set_text_column(PlaceStore.Columns.NAME);
+        let textCell = completion.get_cells()[0];
+        textCell.xpad = textCell.ypad = 4;
+
+        completion.connect('match-selected', (function(completion, model, iter) {
+            let place = model.get_value(iter, PlaceStore.Columns.PLACE);
+
+            this.mapView.showNGotoLocation(place.location);
+        }).bind(this));
+
+        this._searchEntry.set_completion(completion);
     },
 
     _initActions: function() {
@@ -217,9 +251,11 @@ const MainWindow = new Lang.Class({
 
     _onSearchPopupSelected: function(widget, iter) {
         let model = this._searchPopup.getModel();
-        let location = model.get_value(iter, SearchResults.COL_LOCATION);
+        let place = model.get_value(iter, SearchResults.COL_PLACE);
 
-        this.mapView.showNGotoLocation(location);
+        this.mapView.showNGotoLocation(place.location);
+
+        this._placeStore.addRecent(place);
         this._searchPopup.hide();
     },
 
@@ -274,9 +310,9 @@ const MainWindow = new Lang.Class({
 
             model.set(iter,
                       [SearchResults.COL_DESCRIPTION,
-                       SearchResults.COL_LOCATION],
+                       SearchResults.COL_PLACE],
                       [description,
-                       location]);
+                       place]);
 
             if (icon !== null) {
                 Utils.load_icon(icon, _PLACE_ICON_SIZE, function(pixbuf) {
